@@ -1,4 +1,3 @@
-use std::convert::identity;
 use std::iter::{self, FromIterator};
 use std::{slice, vec};
 
@@ -7,31 +6,27 @@ use crate::{NonSortedIntegers, RoaringBitmap};
 
 /// An iterator for `RoaringBitmap`.
 pub struct Iter<'a> {
-    inner: iter::FlatMap<
-        slice::Iter<'a, Container>,
-        &'a Container,
-        fn(&'a Container) -> &'a Container,
-    >,
+    inner: iter::Flatten<slice::Iter<'a, Container>>,
     size_hint: u64,
 }
 
 /// An iterator for `RoaringBitmap`.
 pub struct IntoIter {
-    inner: iter::FlatMap<vec::IntoIter<Container>, Container, fn(Container) -> Container>,
+    inner: iter::Flatten<vec::IntoIter<Container>>,
     size_hint: u64,
 }
 
 impl Iter<'_> {
     fn new(containers: &[Container]) -> Iter {
-        let size_hint = containers.iter().map(|c| c.len).sum();
-        Iter { inner: containers.iter().flat_map(identity as _), size_hint }
+        let size_hint = containers.iter().map(|c| c.len()).sum();
+        Iter { inner: containers.iter().flatten(), size_hint }
     }
 }
 
 impl IntoIter {
     fn new(containers: Vec<Container>) -> IntoIter {
-        let size_hint = containers.iter().map(|c| c.len).sum();
-        IntoIter { inner: containers.into_iter().flat_map(identity as _), size_hint }
+        let size_hint = containers.iter().map(|c| c.len()).sum();
+        IntoIter { inner: containers.into_iter().flatten(), size_hint }
     }
 }
 
@@ -52,6 +47,20 @@ impl Iterator for Iter<'_> {
     }
 }
 
+impl DoubleEndedIterator for Iter<'_> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.size_hint = self.size_hint.saturating_sub(1);
+        self.inner.next_back()
+    }
+}
+
+#[cfg(target_pointer_width = "64")]
+impl ExactSizeIterator for Iter<'_> {
+    fn len(&self) -> usize {
+        self.size_hint as usize
+    }
+}
+
 impl Iterator for IntoIter {
     type Item = u32;
 
@@ -69,6 +78,20 @@ impl Iterator for IntoIter {
     }
 }
 
+impl DoubleEndedIterator for IntoIter {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.size_hint = self.size_hint.saturating_sub(1);
+        self.inner.next_back()
+    }
+}
+
+#[cfg(target_pointer_width = "64")]
+impl ExactSizeIterator for IntoIter {
+    fn len(&self) -> usize {
+        self.size_hint as usize
+    }
+}
+
 impl RoaringBitmap {
     /// Iterator over each value stored in the RoaringBitmap, guarantees values are ordered by value.
     ///
@@ -184,13 +207,31 @@ impl RoaringBitmap {
         &mut self,
         iterator: I,
     ) -> Result<u64, NonSortedIntegers> {
-        let mut count = 0;
+        // Name shadowed to prevent accidentally referencing the param
+        let mut iterator = iterator.into_iter();
+
+        let mut prev = match (iterator.next(), self.max()) {
+            (None, _) => return Ok(0),
+            (Some(first), Some(max)) if first <= max => {
+                return Err(NonSortedIntegers { valid_until: 0 })
+            }
+            (Some(first), _) => first,
+        };
+
+        // It is now guaranteed that so long as the values of the iterator are
+        // monotonically increasing they must also be the greatest in the set.
+
+        self.push_unchecked(prev);
+
+        let mut count = 1;
 
         for value in iterator {
-            if self.push(value) {
-                count += 1;
-            } else {
+            if value <= prev {
                 return Err(NonSortedIntegers { valid_until: count });
+            } else {
+                self.push_unchecked(value);
+                prev = value;
+                count += 1;
             }
         }
 
