@@ -1,5 +1,7 @@
+mod arbitrary;
 mod container;
 mod fmt;
+mod proptests;
 mod store;
 mod util;
 
@@ -15,7 +17,7 @@ use self::cmp::Pairs;
 pub use self::iter::IntoIter;
 pub use self::iter::Iter;
 
-/// A compressed bitmap using the [Roaring bitmap compression scheme](http://roaringbitmap.org).
+/// A compressed bitmap using the [Roaring bitmap compression scheme](https://roaringbitmap.org/).
 ///
 /// # Examples
 ///
