@@ -1,9 +1,11 @@
 use bytemuck::cast_slice_mut;
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
+use std::convert::{Infallible, TryFrom};
+use std::error::Error;
 use std::io;
 
 use super::container::Container;
-use super::store::Store;
+use crate::bitmap::store::{ArrayStore, BitmapStore, Store};
 use crate::RoaringBitmap;
 
 const SERIAL_COOKIE_NO_RUNCONTAINER: u32 = 12346;
@@ -32,7 +34,7 @@ impl RoaringBitmap {
             .containers
             .iter()
             .map(|container| match container.store {
-                Store::Array(ref values) => 8 + values.len() * 2,
+                Store::Array(ref values) => 8 + values.len() as usize * 2,
                 Store::Bitmap(..) => 8 + 8 * 1024,
             })
             .sum();
@@ -64,7 +66,7 @@ impl RoaringBitmap {
 
         for container in &self.containers {
             writer.write_u16::<LittleEndian>(container.key)?;
-            writer.write_u16::<LittleEndian>((container.len - 1) as u16)?;
+            writer.write_u16::<LittleEndian>((container.len() - 1) as u16)?;
         }
 
         let mut offset = 8 + 8 * self.containers.len() as u32;
@@ -83,12 +85,12 @@ impl RoaringBitmap {
         for container in &self.containers {
             match container.store {
                 Store::Array(ref values) => {
-                    for &value in values {
+                    for &value in values.iter() {
                         writer.write_u16::<LittleEndian>(value)?;
                     }
                 }
-                Store::Bitmap(ref values) => {
-                    for &value in values.iter() {
+                Store::Bitmap(ref bits) => {
+                    for &value in bits.as_array() {
                         writer.write_u64::<LittleEndian>(value)?;
                     }
                 }
@@ -99,8 +101,10 @@ impl RoaringBitmap {
     }
 
     /// Deserialize a bitmap into memory from [the standard Roaring on-disk
-    /// format][format].  This is compatible with the official C/C++, Java and
-    /// Go implementations.
+    /// format][format]. This is compatible with the official C/C++, Java and
+    /// Go implementations. This method checks that all of the internal values
+    /// are valid. If deserializing from a trusted source consider
+    /// [RoaringBitmap::deserialize_unchecked_from]
     ///
     /// [format]: https://github.com/RoaringBitmap/RoaringFormatSpec
     ///
@@ -116,7 +120,49 @@ impl RoaringBitmap {
     ///
     /// assert_eq!(rb1, rb2);
     /// ```
-    pub fn deserialize_from<R: io::Read>(mut reader: R) -> io::Result<RoaringBitmap> {
+    pub fn deserialize_from<R: io::Read>(reader: R) -> io::Result<RoaringBitmap> {
+        RoaringBitmap::deserialize_from_impl(reader, ArrayStore::try_from, BitmapStore::try_from)
+    }
+
+    /// Deserialize a bitmap into memory from [the standard Roaring on-disk
+    /// format][format]. This is compatible with the official C/C++, Java and
+    /// Go implementations. This method is memory safe but will not check if
+    /// the data is a valid bitmap.
+    ///
+    /// [format]: https://github.com/RoaringBitmap/RoaringFormatSpec
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use roaring::RoaringBitmap;
+    ///
+    /// let rb1: RoaringBitmap = (1..4).collect();
+    /// let mut bytes = vec![];
+    /// rb1.serialize_into(&mut bytes).unwrap();
+    /// let rb2 = RoaringBitmap::deserialize_unchecked_from(&bytes[..]).unwrap();
+    ///
+    /// assert_eq!(rb1, rb2);
+    /// ```
+    pub fn deserialize_unchecked_from<R: io::Read>(reader: R) -> io::Result<RoaringBitmap> {
+        RoaringBitmap::deserialize_from_impl::<R, _, Infallible, _, Infallible>(
+            reader,
+            |values| Ok(ArrayStore::from_vec_unchecked(values)),
+            |len, values| Ok(BitmapStore::from_unchecked(len, values)),
+        )
+    }
+
+    fn deserialize_from_impl<R, A, AErr, B, BErr>(
+        mut reader: R,
+        a: A,
+        b: B,
+    ) -> io::Result<RoaringBitmap>
+    where
+        R: io::Read,
+        A: Fn(Vec<u16>) -> Result<ArrayStore, AErr>,
+        AErr: Error + Send + Sync + 'static,
+        B: Fn(u64, Box<[u64; 1024]>) -> Result<BitmapStore, BErr>,
+        BErr: Error + Send + Sync + 'static,
+    {
         let (size, has_offsets) = {
             let cookie = reader.read_u32::<LittleEndian>()?;
             if cookie == SERIAL_COOKIE_NO_RUNCONTAINER {
@@ -152,15 +198,18 @@ impl RoaringBitmap {
                 let mut values = vec![0; len as usize];
                 reader.read_exact(cast_slice_mut(&mut values))?;
                 values.iter_mut().for_each(|n| *n = u16::from_le(*n));
-                Store::Array(values)
+                let array = a(values).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
+                Store::Array(array)
             } else {
                 let mut values = Box::new([0; 1024]);
                 reader.read_exact(cast_slice_mut(&mut values[..]))?;
                 values.iter_mut().for_each(|n| *n = u64::from_le(*n));
-                Store::Bitmap(values)
+                let bitmap =
+                    b(len, values).map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
+                Store::Bitmap(bitmap)
             };
 
-            containers.push(Container { key, len, store });
+            containers.push(Container { key, store });
         }
 
         Ok(RoaringBitmap { containers })
