@@ -2,12 +2,14 @@
 //! defined as a [Java library][roaring-java] and described in [_Better bitmap
 //! performance with Roaring bitmaps_][roaring-paper].
 //!
-//! [Rust]: https://rust-lang.org
-//! [Roaring bitmap]: http://roaringbitmap.org
+//! [Rust]: https://www.rust-lang.org/
+//! [Roaring bitmap]: https://roaringbitmap.org/
 //! [roaring-java]: https://github.com/lemire/RoaringBitmap
-//! [roaring-paper]: http://arxiv.org/pdf/1402.6407v4
+//! [roaring-paper]: https://arxiv.org/pdf/1402.6407v4
 
+#![cfg_attr(feature = "simd", feature(portable_simd))]
 #![warn(missing_docs)]
+#![warn(unsafe_op_in_unsafe_fn)]
 #![warn(variant_size_differences)]
 #![allow(unknown_lints)] // For clippy
 
@@ -16,7 +18,7 @@ extern crate byteorder;
 use std::error::Error;
 use std::fmt;
 
-/// A compressed bitmap using the [Roaring bitmap compression scheme](http://roaringbitmap.org).
+/// A compressed bitmap using the [Roaring bitmap compression scheme](https://roaringbitmap.org/).
 pub mod bitmap;
 
 /// A compressed bitmap with u64 values.  Implemented as a `BTreeMap` of `RoaringBitmap`s.
