@@ -19,6 +19,12 @@ impl<'a> Iterator for To64Iter<'a> {
     }
 }
 
+impl DoubleEndedIterator for To64Iter<'_> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.inner.next_back().map(|n| util::join(self.hi, n))
+    }
+}
+
 fn to64iter<'a>(t: (&'a u32, &'a RoaringBitmap)) -> To64Iter<'a> {
     To64Iter { hi: *t.0, inner: t.1.iter() }
 }
@@ -35,6 +41,12 @@ impl Iterator for To64IntoIter {
     }
 }
 
+impl DoubleEndedIterator for To64IntoIter {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.inner.next_back().map(|n| util::join(self.hi, n))
+    }
+}
+
 fn to64intoiter(t: (u32, RoaringBitmap)) -> To64IntoIter {
     To64IntoIter { hi: t.0, inner: t.1.into_iter() }
 }
@@ -95,6 +107,20 @@ impl<'a> Iterator for Iter<'a> {
     }
 }
 
+impl DoubleEndedIterator for Iter<'_> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.size_hint = self.size_hint.saturating_sub(1);
+        self.inner.next_back()
+    }
+}
+
+#[cfg(target_pointer_width = "64")]
+impl ExactSizeIterator for Iter<'_> {
+    fn len(&self) -> usize {
+        self.size_hint as usize
+    }
+}
+
 impl Iterator for IntoIter {
     type Item = u64;
 
@@ -112,6 +138,20 @@ impl Iterator for IntoIter {
     }
 }
 
+impl DoubleEndedIterator for IntoIter {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.size_hint = self.size_hint.saturating_sub(1);
+        self.inner.next_back()
+    }
+}
+
+#[cfg(target_pointer_width = "64")]
+impl ExactSizeIterator for IntoIter {
+    fn len(&self) -> usize {
+        self.size_hint as usize
+    }
+}
+
 impl RoaringTreemap {
     /// Iterator over each value stored in the RoaringTreemap, guarantees values are ordered by
     /// value.
@@ -255,13 +295,28 @@ impl RoaringTreemap {
         &mut self,
         iterator: I,
     ) -> Result<u64, NonSortedIntegers> {
-        let mut count = 0;
+        let mut iterator = iterator.into_iter();
+        let mut prev = match (iterator.next(), self.max()) {
+            (None, _) => return Ok(0),
+            (Some(first), Some(max)) if first <= max => {
+                return Err(NonSortedIntegers { valid_until: 0 })
+            }
+            (Some(first), _) => first,
+        };
 
+        // It is now guaranteed that so long as the values of the iterator are
+        // monotonically increasing they must also be the greatest in the set.
+
+        self.push_unchecked(prev);
+
+        let mut count = 1;
         for value in iterator {
-            if self.push(value) {
-                count += 1;
-            } else {
+            if value <= prev {
                 return Err(NonSortedIntegers { valid_until: count });
+            } else {
+                self.push_unchecked(value);
+                prev = value;
+                count += 1;
             }
         }
 
