@@ -19,10 +19,8 @@ fn bitmap() {
 
 #[test]
 fn arrays() {
-    let original = ((0..2000)
-        .chain(1_000_000..1_002_000)
-        .chain(2_000_000..2_001_000))
-    .collect::<RoaringTreemap>();
+    let original = ((0..2000).chain(1_000_000..1_002_000).chain(2_000_000..2_001_000))
+        .collect::<RoaringTreemap>();
     let clone = original.clone();
 
     assert_eq!(clone, original);
@@ -30,10 +28,8 @@ fn arrays() {
 
 #[test]
 fn bitmaps() {
-    let original = ((0..6000)
-        .chain(1_000_000..1_012_000)
-        .chain(2_000_000..2_010_000))
-    .collect::<RoaringTreemap>();
+    let original = ((0..6000).chain(1_000_000..1_012_000).chain(2_000_000..2_010_000))
+        .collect::<RoaringTreemap>();
     let clone = original.clone();
 
     assert_eq!(clone, original);
